#include "parser.h"

#include <io.h>
#include <fcntl.h>

const char*Parser::errstr(err_t err) {
 switch(err) {
  case OK		: return "OK";
  case ERR_SYSTEM	: return "System Error";
  case ERR_INVALID_FILE	: return "Invalid File";
  case ERR_WRONLY	: return "Parser can only write";
  case ERR_RDONLY	: return "Parser can only read";
  default		: return "Unknown Error";
 }
}

Parser::err_t ParserBin::open(const char*filename, bool write) {
 wr=write;
 if (write) {
  if (filename[0] == '-' && !filename[1]) fd = 1;
  else{
#ifdef _WIN32
# ifdef UNICODE	// filename may contain non-ASCII characters, so use wide-char open function
   int len=MultiByteToWideChar(CP_UTF8,0,filename,-1,0,0);
   wchar_t*n=new wchar_t[len];
   MultiByteToWideChar(CP_UTF8,0,filename,-1,n,len);
   fd=_wopen(n,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY);
   delete[]n;
# elif defined(_WIN64)
   fd = old_msvcrt::_open(filename,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY);
# else
   fd = _open(filename,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,0);
# endif
#else
   fd = _open(filename,O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
#endif
  }
 }else{
  if (filename[0] == '-' && !filename[1]) fd = 0;
  else {
#ifdef _WIN32
# ifdef UNICODE
   int len=MultiByteToWideChar(CP_UTF8,0,filename,-1,0,0);
   wchar_t*n=new wchar_t[len];
   MultiByteToWideChar(CP_UTF8,0,filename,-1,n,len);
   fd=_wopen(n,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY);
   delete[]n;
# elif defined(_WIN64)
   fd = old_msvcrt::_open(filename,O_RDONLY|O_BINARY);
#else
   fd = _open(filename,O_RDONLY|O_BINARY);
#endif
#else
   fd = _open(filename,O_RDONLY);
#endif
  }
 }
 if (fd<0) return ERR_SYSTEM;
 _lseek(fd,0,SEEK_END);
 filesize=_tell(fd);
 _lseek(fd,0,0);
 return OK;
}

ParserBin::~ParserBin() {
 if (fd>=0) _close(fd);
}

Parser::err_t ParserBin::read(void *data, unsigned&len) {
 if (wr) return ERR_WRONLY;
 unsigned left = len;
 uint8_t*d = reinterpret_cast<uint8_t*>(data);

 while(left > 0) {
  int r = _read(fd, d, left);
		/* If there is no data to read at all, return OK, but with zero read */
  if (!r && left == len) {
   len = 0;
   return OK;
  }
  if (r<=0) return ERR_SYSTEM;
  left -= r;
  d += r;
 }
 len -= left;
 return OK;
}

Parser::err_t ParserBin::write(const void *data, unsigned len) {
 if (!wr) return ERR_RDONLY;
 const uint8_t*d = reinterpret_cast<const uint8_t*>(data);
 while(len > 0) {
  int r = _write(fd, d, len);
  if (r<1) return ERR_SYSTEM;
  filesize += r;
  len  -= r;
  d += r;
 }
 return OK;
}
